;  ------------------------------------------------------------------

Proc Lookup_Select(edit_is_allowed)

;  Manages a lookup-help function

;  Parameter:
;     edit_is_allowed -- True, if the user may press [F9] to
;                              edit the lookup table;
;                        False, if [F9] is illegal

Private
   p1,p2,      ;prompt lines
   x           ;the keystroke pressed by the user

;  Press [F1] and see if it places us into lookup-help

Help                                      ;press [F1]
If helpmode()<>"LookupHelp" then          ;no, it's not lookup help
   If helpmode()="Help" then Esc Endif    ;it's regular help; escape from it
   Beep                                   ;user can't press [F1] here
   Return "[F1] is not available for this field."
Endif

;  Set up the lookup-help environment

p1 = "Move to the record you want to select"
If edit_is_allowed then
   p2 = "Press [F2] to select the record; [Esc] to cancel; [F9] to edit"
Else
   p2 = "Press [F2] to select the record; [Esc] to cancel"
Endif
Prompt p1,p2
Echo Normal

;  Let the user press keys until pressing one which exits from lookup-help

While helpmode()<>"None"               ;while still in look-up help
   x = getchar()                       ;let the user press a key
   Switch
      Case x=15 or x=-24 or x=-59:     ;[Ctrl-O], [Alt-O], [F1] are illegal
         Beep                          ;... so ignore them
      Case x=-67 and edit_is_allowed:  ;[F9] to edit the lookup table
         Lookup_Edit()                 ;edit the lookup table         
         Prompt p1,p2
      Case x=-67:       Beep           ;[F9] when it's not allowed
      Otherwise:        KeyPress x     ;perform it
   EndSwitch
EndWhile

;  All done

Echo Off
Prompt
Return ""

EndProc

;  ------------------------------------------------------------------

Proc Lookup_Edit()

;  Allows the editing of a lookup table from within lookup-help.
;  In order to implement this feature, you must be in edit or co-edit
;  mode, and the lookup table must be an image on the workspace.

Private
   lut,           ;lookup table name
   lutkey,        ;key value within the lookup table
   origt,         ;original table name
   wasformview    ;whether the original table is in form view

;  Remember the current location of the cursor within the lookup table

Echo Off                   ;don't show the user what's going on
lut = upper(table())       ;get the lookup-table name
CtrlHome Right             ;move to the key field column
lutkey = []                ;remember the current key value

;  Switch from lookup mode to the lookup table's image

Esc                           ;escape from lookup-help
If helpmode()<>"None" then Return "" Endif      ;the escape failed
origt = table()               ;get the original table name
wasformview = isformview()    ;remember whether we're in a form here
MoveTo lut                    ;switch to the lookup table image
FirstShow                     ;make it first on the screen

;  Let the user edit the lookup table

If upper(table())=lut then    ;we moved there successfully
   CtrlHome Right             ;move to the key field column
   Locate lutkey              ;move to the same record
   Prompt "Edit the "+lut+" lookup table",
          "Press [F2] when done"
   Wait Table Until "F2"
   CtrlHome Right             ;move to the key field column
   lutkey = []                ;remember the current key value

;  Test for a duplicate key problem resulting from the Wait Table

   If Not IsBlank(lutkey) Then
      LockRecord                 ;try to lock the current record
      If not retval and errorcode()=53 then     ;it's a duplicate key
         Undo Undo               ;remove the duplicate
      Endif
      UnLockRecord               ;unlock the record
   Else
      Del
   Endif
Endif

;  Return to the original table, and resume lookup-help

MoveTo origt                  ;return to the original table
FirstShow                     ;make it first on the screen
If wasformview and not isformview() then     ;we were in a form originally
   FormKey                    ;return to the form
Endif
Help                          ;return to lookup-help
If helpmode()="LookupHelp" then          ;we're back in lookup-help
   Locate lutkey              ;return to the original record
Else                          ;failed to get back into lookup-help
   Esc                        ;so switch back
Endif

Echo Normal                   ;resume echoing

EndProc

;  ------------------------------------------------------------------

;  A simple script to test the Lookup_Select() and Lookup_Edit()
;  procedures.


View "Customer"      ;place the lookup table on the workspace
CoEdit "Rental"      ;view the main table, begin co-editing
FormKey              ;switch to the form
wait_msg = ""        ;no message yet

While True           ;repeat the "Wait" command till the user presses [F2]

   Wait Record       ;let the user edit the record
      Prompt "Edit the Rental Table",
             "[F2]=Do-it!  [F1]=LookupHelp"
      Message wait_msg
      Until "F2","F1","PgUp","PgDn","CtrlPgUp","CtrlPgDn",
            "Home","End","Ins"

   user_key = retval       ;remember the key that was pressed
   wait_msg = ""           ;clear the error message, if any

   Switch
      Case user_key="F2":  QuitLoop             ;all done
      Case user_key="F1": 
         edit_is_allowed =             ;editing the lookup table is allowed
            (field()="Customer #")     ;on the Customer # field
         Lookup_Select(edit_is_allowed)         ;the [F1] lookup process
         wait_msg = retval                      ;remember the error message
      Otherwise:           KeyPress user_key    ;press the key for the user
   EndSwitch
EndWhile

Do_it!         ;end the edit process

;  ------------------------------------------------------------------

